/*************************************************************03.09.2001*/
/* eqn_quintic: solution of a 5th order algebraic equation		*/
/*									*/
/* method: The 1st root is obtained with a regula falsi iteration.	*/
/*	   Deflation of the polynomial with this root yields a 4th	*/
/*	   order polynomial, which is then solved analytically.		*/
/*									*/
/* Equations of lesser degree are solved by the appropriate formulas.	*/
/* The solutions are arranged in ascending order.			*/
/************************************************************************/
/* a		(i)  array containing the polynomial coefficients
   x		(o)  array vector
   RESULT	(o)  number of valid solutions				*/
/*======================================================================*/
#include <math.h>
#include <mathc.h>

#define EPS		1.0E-10

static double poly5(double x, const double *ac);

int eqn_quintic(const double *a, double *x) {
  int i, nreal;
  double r, xa, xb, ya, yb, h, ac[6];

  if (a[5] == 0.0) return (eqn_quartic(a, x));

  for (i = 0, r = 1.0, h = 1.0/a[5]; i <= 4; i++) {
    ac[i] = a[i]*h;			/* reduce polynomial */
    r = MAX(r, fabs(ac[i]));
  }
  ac[5] = 1.0;
  r += 1.0;				/* radius of root circle */

  xa = -r;				/* find 1st root */
  ya = poly5(xa, ac);
  xb = r;
  yb = poly5(xb, ac);
  x[4] = regula_falsi(poly5, ac, xa, xb, ya, yb, EPS*r);

  ac[4] = a[5];				/* deflation */
  for (i = 3; i >= 0; i--) ac[i] = a[i+1] + x[4]*ac[i+1];

  nreal = 1 + eqn_quartic(ac, x);

  if (nreal < 5) x[nreal-1] = x[4];
  for (i = nreal-2; i >= 0; i--) {	/* sort results */
    if (x[i+1] > x[i])
      break;
    else
      SWAP(x[i], x[i+1], h);
  }
  return (nreal);
}

/************************************************************************/
/* poly5: 5th order polynomial						*/
/************************************************************************/
/* x		(i)  abscissa
   ac		(i)  coefficients
   RESULT	(o)  value of polynomial at x				*/
/*======================================================================*/
static double poly5(double x, const double *ac) {
  int i;
  double h;

  for (i = 4, h = ac[5]; i >= 0; i--) h = h*x + ac[i];
  return (h);
}
