/*************************************************************20.12.2001*/
/* vector operations							*/
/************************************************************************/
#include <stdlib.h>
#include <math.h>
#include <mathc.h>

/*************************************************************20.12.2001*/
/* vector_ops: simple vector operations					*/
/************************************************************************/
/* v1, v2	(i)  input vectors					*/
/* v3		(o)  output vector					*/
/* a		(i)  scalar operand					*/
/* f(x)		(i)  1-dimensional function				*/
/* RESULT	(o)  error code: 0 = no error				*/
/*				 1 = incompatible vector dimensions	*/
/*		     Note: Only vector sizes are compared, not index	*/
/*		     ranges.						*/
/*									*/
/* operations:	+                              subroutine: add		*/
/*		-                                          sub		*/
/*		* (element by element)                     mul		*/
/*		/                                          div		*/
/*		scalar multiplication v3=v1*a              smul		*/
/* 		scalar addition                            sadd		*/
/*		scalar assignment v3=(a)                   set		*/
/*		copy                                       copy		*/
/*		function evaluation v3=f(v1)               vfunc	*/
/*======================================================================*/
int dvector_add(const dvector *v1, const dvector *v2, dvector *v3) {
  double *p1, *p2, *p3, *pmax;

  if (v1->n != v2->n || v1->n != v3->n) return (1);
  p1 = v1->data;
  p2 = v2->data;
  p3 = v3->data;
  pmax = p3 + v3->n;
  while (p3 < pmax) *p3++ = *p1++ + *p2++;
  return (0);
}
 
int dvector_sub(const dvector *v1, const dvector *v2, dvector *v3) {
  double *p1, *p2, *p3, *pmax;

  if (v1->n != v2->n || v1->n != v3->n) return (1);
  p1 = v1->data;
  p2 = v2->data;
  p3 = v3->data;
  pmax = p3 + v3->n;
  while (p3 < pmax) *p3++ = *p1++ - *p2++;
  return (0);
}
 
int dvector_mul(const dvector *v1, const dvector *v2, dvector *v3) {
  double *p1, *p2, *p3, *pmax;

  if (v1->n != v2->n || v1->n != v3->n) return (1);
  p1 = v1->data;
  p2 = v2->data;
  p3 = v3->data;
  pmax = p3 + v3->n;
  while (p3 < pmax) *p3++ = *p1++ * *p2++;
  return (0);
}
 
int dvector_div(const dvector *v1, const dvector *v2, dvector *v3) {
  double *p1, *p2, *p3, *pmax;

  if (v1->n != v2->n || v1->n != v3->n) return (1);
  p1 = v1->data;
  p2 = v2->data;
  p3 = v3->data;
  pmax = p3 + v3->n;
  while (p3 < pmax) *p3++ = *p1++ / *p2++;
  return (0);
}
 
int dvector_smul(const dvector *v1, double a, dvector *v3) {
  double *p1, *p3, *pmax;

  if (v1->n != v3->n) return (1);
  p1 = v1->data;
  p3 = v3->data;
  pmax = p3 + v3->n;
  while (p3 < pmax) *p3++ = *p1++ * a;
  return (0);
}
 
int dvector_sadd(const dvector *v1, double a, dvector *v3) {
  double *p1, *p3, *pmax;

  if (v1->n != v3->n) return (1);
  p1 = v1->data;
  p3 = v3->data;
  pmax = p3 + v3->n;
  while (p3 < pmax) *p3++ = *p1++ + a;
  return (0);
}
 
int dvector_set(double a, dvector *v3) {
  double *p3, *pmax;

  p3 = v3->data;
  pmax = p3 + v3->n;
  while (p3 < pmax) *p3++ = a;
  return (0);
}
 
int dvector_copy(const dvector *v1, dvector *v3) {
  double *p1, *p3, *pmax;

  if (v1->n != v3->n) return (1);
  p1 = v1->data;
  p3 = v3->data;
  pmax = p3 + v3->n;
  while (p3 < pmax) *p3++ = *p1++;
  return (0);
}

int dvector_func(double (*f)(), const dvector *v1, dvector *v3) {
  double *p1, *p3, *pmax;

  if (v1->n != v3->n) return (1);
  p1 = v1->data;
  p3 = v3->data;
  pmax = p3 + v3->n;
  while (p3 < pmax) *p3++ = (*f)(*p1++);
  return (0);
}

double dvector_dot(const dvector *v1, const dvector *v2) {
  double *p1, *p2, *pmax, a;

  p1 = v1->data;
  pmax = p1 + v1->n;
  p2 = v2->data;
  a = 0.0;
  while (p1 < pmax) a += *p1++ * *p2++;	/* assume that v1->n == v2->n */
  return (a);
}

/*************************************************************03.09.2001*/
/* dvector_create: allocate a double vector with user-defined subscript	*/
/*	    range: v[nl..nh]						*/
/************************************************************************/
/* v		(o)  vector to be allocated
   nl, nh	(i)  lower and upper indices
   RESULT	(o)  error code: 0 = no error
				 1 = memory allocation failed		*/
/*======================================================================*/
int dvector_create(dvector *v, int nl, int nh) {
  v->nl = nl;
  v->nh = nh;
  v->n = nh - nl + 1;

  v->data = (double *) valloc((size_t) (v->n * sizeof(double)));
  if (v->data == NULL) return (1);
  v->a = v->data - nl;
  return (0);
}

/*************************************************************03.09.2001*/
/* dvector_free: deallocate a double vector				*/
/************************************************************************/
/* v		(i)  vector to be deallocated				*/
/*======================================================================*/
void dvector_free(dvector *v) {
  free((char *) v->data);
}

/*************************************************************03.09.2001*/
/* ivector_create: allocate an int vector with user-defined subscript	*/
/*	    range: v[nl..nh]						*/
/************************************************************************/
/* v		(o)  vector to be allocated
   nl, nh	(i)  lower and upper indices
   RESULT	(o)  error code: 0 = no error
				 1 = memory allocation failed		*/
/*======================================================================*/
int ivector_create(ivector *v, int nl, int nh) {
  v->nl = nl;
  v->nh = nh;
  v->n = nh - nl + 1;

  v->data = (int *) valloc((size_t) (v->n * sizeof(int)));
  if (v->data == NULL) return (1);
  v->a = v->data - nl;
  return (0);
}

/*************************************************************03.09.2001*/
/* ivector_free: deallocate an integer vector				*/
/************************************************************************/
/* v		(i)  vector to be deallocated				*/
/*======================================================================*/
void ivector_free(ivector *v) {
  free((char *) v->data);
}

/*************************************************************03.09.2001*/
/* dvector_overlay: overlay a double array with a vector		*/
/*									*/
/* This is almost the same as dvector_create(), except that here no 	*/
/* dynamic memory allocations are used.					*/
/*									*/
/* usage:								*/
/*   dvector a;								*/
/*   double aspace[12];							*/
/*   dvector_overlay(&a, aspace, -2, 9);				*/
/*   h = a.a[-2] + a.a[9];						*/
/*									*/
/* WARNING: The array boundary ranges must match the array size. There	*/
/*	    is no way to check this at runtime!				*/
/************************************************************************/
/* v		(o)   dvector to be generated
   a		(i)   array to be organized as a vector
   nl, nh	(i)   lower and upper boundaries			*/
/*======================================================================*/
void dvector_overlay(dvector *m, double *a, int nl, int nh) {
  m->nl = nl;
  m->nh = nh;
  m->n = nh - nl + 1;
  m->data = a;
  m->a = m->data - nl;
}

/*************************************************************03.09.2001*/
/* ivector_overlay: overlay an integer array with a vector		*/
/*									*/
/* usage: see dvector_overlay()						*/
/************************************************************************/
/* v		(o)   ivector to be generated
   a		(i)   array to be organized as a vector
   nl, nh	(i)   lower and upper boundaries			*/
/*======================================================================*/
void ivector_overlay(ivector *m, int *a, int nl, int nh) {
  m->nl = nl;
  m->nh = nh;
  m->n = nh - nl + 1;
  m->data = a;
  m->a = m->data - nl;
}
